{extends "../layout.tpl"}
{$page="admin.reports"}

{block 'title'}
    {$lang->relatory}
{/block}

{block "content"}
    <div>
        {if count($reports) === 0}
            <p>{$lang->no_relatory}</p>
        {else}
            <div class="panel panel-default">
                <div class="table-responsive">
                    <table class="table table-hover">
                        <thead>
                            <tr>
                                <th>#</th>
                                <th>{$lang->date}</th>
                                <th>{$lang->status}</th>
                                <th>{$lang->platform}</th>
                                <th>{$lang->notification}</th>
                                <th>{$lang->client}</th>
                                <th>{$lang->category}</th>
                            </tr>
                        </thead>
                        <tbody>
                            {foreach from=$reports item=$report}
                                <tr>
                                    <td>{$report->id}</td>
                                    <td>
                                        <span
                                            data-toggle="tooltip"
                                            data-placement="left"
                                            title="{$report->created_at|date_format:"%d/%m/%Y às %H:%M"}"
                                        >
                                            {$report->created_at|date_format:"%d/%m/%Y"}
                                        </span>
                                    </td>
                                    <td>
                                        <span class="label label-{($report->status === 'sent') ? 'success' : 'danger'}">
                                            {($report->status === 'sent') ? {$lang->sent} : {$lang->error}}
                                        </span>
                                    </td>
                                    <td>
                                        {($report->platform === 'wp') ? 'WhatsApp' : 'Chatwoot'}
                                        {if $report->channel}
                                            {$lang->by} {($report->channel === 'wp') ? 'WhatsApp' : ''}
                                        {/if}
                                    </td>
                                    <td>{$report->notification}</td>
                                    <td>
                                        {if $report->client_id}
                                            <a
                                                target="_blank"
                                                href="clientssummary.php?userid={$report->client_id}"
                                            >
                                                #{$report->client_id}
                                            </a>
                                        {/if}
                                    </td>
                                    <td>
                                        {if !empty($report->category) && !empty($report->category_id)}
                                            {if $report->category === 'invoice'}
                                                {assign "category_link" "invoices.php?action=edit&id="}
                                                {assign "category_label" {$lang->invoice}}
                                            {elseif $report->category === 'order'}
                                                {assign "category_link" "orders.php?action=view&id="}
                                                {assign "category_label" {$lang->order}}
                                            {elseif $report->category === 'ticket'}
                                                {assign "category_link" "supporttickets.php?action=view&id="}
                                                {assign "category_label" {$lang->ticket}}
                                            {elseif $report->category === 'domain'}
                                                {assign "category_link" "clientsdomains.php?userid={$report->client_id}&id={$report->category_id}"}
                                                {assign "category_label" {$lang->domain}}
                                            {/if}
                                            <a
                                                target="_blank"
                                                href="{$category_link}{$report->category_id}"
                                            >
                                                {$category_label} #{$report->category_id}
                                            </a>
                                        {/if}
                                    </td>
                                </tr>
                            {/foreach}
                        </tbody>
                    </table>
                </div>
            </div>

            {assign "totalPages" value=ceil($totalReports / $reportsPerPage)}

            {if $totalPages > 1}
                <nav
                    aria-label="Page navigation"
                    style="text-align: center;"
                >
                    <ul class="pagination">
                        {if $currentPage > 6}
                            <li>
                                <a href="?module=lknhooknotification&c=admin&r=reports&page=1">
                                    {$lang->first_page}
                                </a>
                            </li>
                        {/if}
                        <li
                            {if $currentPage == 1}
                                class="disabled"
                            {/if}
                        >
                            <a
                                {if $currentPage > 1}
                                    href="?module=lknhooknotification&c=admin&r=reports&page={$currentPage - 1}"
                                {/if}
                                aria-label="Previous"
                            >
                                <span aria-hidden="true">&laquo;</span>
                            </a>
                        </li>

                        {if $totalPages >= 15}


                            {for $page=$currentPage - 8 to $currentPage}
                                {if $page > 0}
                                    <li
                                        {if $page == $currentPage}
                                            class="active"
                                        {/if}
                                    >
                                        <a href="?module=lknhooknotification&c=admin&r=reports&page={$page}">{$page}</a>
                                    </li>
                                {/if}
                            {/for}

                            {for $page=$currentPage + 1 to $currentPage + 8}
                                {if $page < $totalPages}
                                    <li
                                        {if $page == $currentPage}
                                            class="active"
                                        {/if}
                                    >
                                        <a href="?module=lknhooknotification&c=admin&r=reports&page={$page}">{$page}</a>
                                    </li>
                                {/if}
                            {/for}


                        {else}
                            {for $page=1 to $totalPages}
                                <li
                                    {if $page == $currentPage}
                                        class="active"
                                    {/if}
                                ><a href="?module=lknhooknotification&c=admin&r=reports&page={$page}">{$page}</a></li>
                            {/for}
                        {/if}

                        <li
                            {if $currentPage >= $totalPages}
                                class="disabled"
                            {/if}
                        >
                            <a
                                {if $currentPage < $totalPages}
                                    href="?module=lknhooknotification&c=admin&r=reports&page={$currentPage + 1}"
                                {/if}
                                aria-label="Next"
                            >
                                <span aria-hidden="true">&raquo;</span>
                            </a>
                        </li>

                        {if $currentPage <= $totalPages - 6}
                            <li>
                                <a href="?module=lknhooknotification&c=admin&r=reports&page={$totalPages}">
                                    {$lang->last_page}
                                </a>
                            </li>
                        {/if}
                    </ul>
                </nav>
            {/if}
        {/if}
    </div>
{/block}
