/* global $ request isEditMode LknHookNotModal ASSOC MESSAGE_TEMPLATES LKN_CSRF_TOKEN */

const headerComponentRoot = document.getElementById('header-component-root')
const headerComponentParamsRoot = document.getElementById('header-component-params-root')
const bodyComponentRoot = document.getElementById('body-component-root')
const footerComponentRoot = document.getElementById('footer-component-root')
const footerComponentParamsRoot = document.getElementById('footer-component-params-root')
const bodyComponentParamsRoot = document.getElementById('body-component-params-root')
const buttonComponentRoot = document.getElementById('button-components-root')
const buttonComponentParamsRoot = document.getElementById('button-component-params-root')
const inputSelectNotification = document.getElementById('notif-code-select')
const inputSelectMessageTemplate = document.getElementById('message-template-select')
const componentsRoot = document.getElementById('components-root')
const btnSaveAssoc = document.getElementById('btn-add-association')
const wpAssociationForm = document.getElementById('wp-association-form')

/**
 * This class is responsible for creating the form that the user interacts with.
 */
class NotificationSetupForm {
  paramMarkerRegex = /{{\s*(\d+)\s*}}/g

  /**
   * @param {HTMLDivElement} headerComponentRoot
   * @param {HTMLDivElement} headerComponentParamsRoot
   * @param {HTMLDivElement} bodyComponentRoot
   * @param {HTMLDivElement} bodyComponentParamsRoot
   * @param {HTMLDivElement} buttonComponentRoot
   * @param {HTMLDivElement} buttonComponentParamsRoot
   * @param {HTMLDivElement} footerComponentRoot
   * @param {HTMLDivElement} footerComponentParamsRoot
   * @param {HTMLButtonElement} btnSaveAssoc
   */
  constructor(
    componentsRoot,
    headerComponentRoot,
    headerComponentParamsRoot,
    bodyComponentRoot,
    bodyComponentParamsRoot,
    buttonComponentRoot,
    buttonComponentParamsRoot,
    footerComponentRoot,
    footerComponentParamsRoot,
    btnSaveAssoc
  ) {
    this.componentsRoot = componentsRoot
    this.headerComponentRoot = headerComponentRoot
    this.headerComponentParamsRoot = headerComponentParamsRoot
    this.bodyComponentRoot = bodyComponentRoot
    this.bodyComponentParamsRoot = bodyComponentParamsRoot
    this.buttonComponentRoot = buttonComponentRoot
    this.buttonComponentParamsRoot = buttonComponentParamsRoot
    this.footerComponentRoot = footerComponentRoot
    this.footerComponentParamsRoot = footerComponentParamsRoot
    this.btnSaveAssoc = btnSaveAssoc
    this.params = []
  }

  setLoadingState (loading) {
    this.btnSaveAssoc.disabled = loading
  }

  /**
   * @param {Object[]} params
   */
  updateNotifParams (params) {
    this.setLoadingState(true)
    this.params = params

    document.querySelectorAll('.param-select').forEach(select => {
      select.disabled = true
      select.innerHTML = ''

      const optionElement = document.createElement('option')

      optionElement.value = ''
      optionElement.textContent = window.lknHn.lang.select_replacement

      select.appendChild(optionElement)

      for (const param of this.params) {
        const optionElement = document.createElement('option')

        optionElement.value = param.name
        optionElement.textContent = param.label

        select.appendChild(optionElement)
      }

      select.disabled = false
    })

    this.setLoadingState(false)
  }

  /**
   * @param {Object} dataset
   *
   * @returns {HTMLSelectElement}
   */
  generateParamsSelect (dataset) {
    const selectElement = document.createElement('select')

    selectElement.required = true

    selectElement.classList.add('form-control', 'param-select')

    Object.assign(selectElement.style, {
      color: 'black',
      width: '170px',
      display: 'inline',
      height: 'min-content',
      padding: '2px',
      borderRadius: '5px'
    })

    Object.assign(selectElement.dataset, dataset)

    const optionElement = document.createElement('option')

    optionElement.value = ''
    optionElement.textContent = window.lknHn.lang.select_replacement

    selectElement.appendChild(optionElement)

    for (const option of this.params) {
      const optionElement = document.createElement('option')

      optionElement.value = option.name
      optionElement.textContent = option.label

      selectElement.appendChild(optionElement)
    }

    return selectElement
  }

  /**
   *
   * @param {String} string
   * @param {HTMLDivElement} paramsRoot
   * @param {String} className
   * @param {Array} assoc
   * @param {Object} dataset
   *
   * @returns {String}
   */
  getComponentMsgWithParamsSelects (string, paramsRoot, className, assoc, dataset) {
    const paramsSelect = this.generateParamsSelect(dataset)

    const bodyMsgWithParams = string.replace(this.paramMarkerRegex, (match, key) => {
      const thisParamsSelect = paramsSelect.cloneNode(true)

      thisParamsSelect.classList.add(className)
      thisParamsSelect.dataset.key = key

      return thisParamsSelect.outerHTML
    })

    paramsRoot.innerHTML += bodyMsgWithParams

    if (assoc && assoc.length > 0) {
      document.querySelectorAll('.' + className).forEach(paramSelect => {
        const key = parseInt(paramSelect.dataset.key)

        const assocForParam = assoc.find(p => parseInt(p.key) === key)

        if (assocForParam) {
          paramSelect.value = assocForParam.value
        }
      })
    }

    return bodyMsgWithParams
  }

  makeHeaderComponent (header) {
    const format = header.format
    let headerAssoc = []

    if (isEditMode) {
      if (!Array.isArray(ASSOC.components.header)) {
        if (ASSOC.components.header.value) {
          headerAssoc = [{ key: 1, value: ASSOC.components.header.value }]
        } else {
          headerAssoc = []
        }
      } else {
        headerAssoc = ASSOC.components.header
      }
    }

    const documentHeaderText = '{{1}}'

    switch (format) {
      case 'DOCUMENT':
        this.getComponentMsgWithParamsSelects(
          documentHeaderText,
          this.headerComponentParamsRoot,
          'header-param',
          headerAssoc,
          { type: 'document' }
        )

        break

      case 'TEXT':
        this.getComponentMsgWithParamsSelects(
          header.text,
          this.headerComponentParamsRoot,
          'header-param',
          headerAssoc,
          { type: 'text' }
        )

        break
      default:
        this.headerComponentParamsRoot.innerHTML = `
        <span class="text-danger">${window.lknHn.lang.unsupported_header_format.replace(':format', format)}</span>
        `

        break
    }

    this.headerComponentRoot.firstElementChild.lastElementChild.innerHTML = format

    this.toggleHeaderComponent(false)
  }

  makeBodyComplement (body) {
    const text = body.text
    const assocs = isEditMode ? ASSOC.components.body : []

    this.getComponentMsgWithParamsSelects(
      text,
      this.bodyComponentParamsRoot,
      'body-param',
      assocs
    )

    this.toggleBodyComponent(false)
  }

  makeButtonComponents (button) {
    const buttons = button.buttons

    const buttonWrapper = (text, index, type, url = '') => {
      let styledButton = '<div class="well" style="margin-bottom: 15px; text-align: center;">'

      if (url) {
        styledButton +=
          `<div style="width: fit-content; font-weight: bold; margin: 0 auto 0; padding: 8px; border-radius: 3px;">
            <span class="label label-primary">${type}</span> ${text}
          </div>
          <span class="button-param-wrapper" data-btn-index="${index}" data-btn-type="${type}">${url}</span>
          `
      } else {
        styledButton += text
      }

      return styledButton
    }

    buttons.forEach((button, btnIndex) => {
      const type = button.type
      const text = button.text
      const url = decodeURI(button.url) ?? ''
      const assocs = isEditMode ? ASSOC.components.button[btnIndex]?.params : []
      const styledButton = buttonWrapper(text, btnIndex, type, url)

      switch (type) {
        case 'URL':
          this.getComponentMsgWithParamsSelects(
            styledButton,
            this.buttonComponentParamsRoot,
            'button-param',
            assocs
          )

          break

        default:
          this.buttonComponentParamsRoot.innerHTML += buttonWrapper(text, btnIndex, type)

          break
      }
    })

    this.toggleButtonComponents(false)
  }

  makeFooterComponent (text) {
    this.footerComponentParamsRoot.innerHTML = text

    this.toggleFooterComponent(false)
  }

  async reset () {
    return await $('#components-root').slideUp('slow', () => {
      this.toggleHeaderComponent()
      this.toggleBodyComponent()
      this.toggleButtonComponents()
      this.toggleFooterComponent()
    }).promise()
  }

  toggleFooterComponent (hide = true) {
    if (hide) {
      this.footerComponentParamsRoot.innerHTML = ''
      this.footerComponentRoot.style.display = 'none'
    } else {
      this.footerComponentRoot.style.display = 'block'
      $('#components-root').slideDown()
    }
  }

  toggleHeaderComponent (hide = true) {
    if (hide) {
      this.headerComponentParamsRoot.innerHTML = ''
      this.headerComponentRoot.style.display = 'none'
    } else {
      this.headerComponentRoot.style.display = 'block'
      $('#components-root').slideDown()
    }
  }

  toggleBodyComponent (hide = true) {
    if (hide) {
      this.bodyComponentParamsRoot.innerHTML = ''
      this.bodyComponentRoot.style.display = 'none'
    } else {
      this.bodyComponentRoot.style.display = 'block'
      $('#components-root').slideDown()
    }
  }

  toggleButtonComponents (hide = true) {
    if (hide) {
      this.buttonComponentParamsRoot.innerHTML = ''
      this.buttonComponentRoot.style.display = 'none'
    } else {
      this.buttonComponentRoot.style.display = 'block'
      $('#components-root').slideDown()
    }
  }
}

/**
 * This class is responsible for converting the HTML fields of the request body
 * to save the association when the user clicks on the "Save" button.
 */
class AssocGenerator {
  /**
   * @param {HTMLSelectElement} headerParamsInputs
   * @param {HTMLSelectElement} headerParamsInputs
   * @param {HTMLSelectElement[]} headerParamsInputs
   * @param {HTMLSelectElement[]} bodyParamsInputs
   * @param {HTMLDivElement[]} buttonParamsInputs
   */
  constructor(
    inputSelectNotification,
    inputSelectMessageTemplate,
    headerParamsInputs,
    bodyParamsInputs,
    buttonParamsInputs
  ) {
    this.inputSelectNotification = inputSelectNotification
    this.inputSelectMessageTemplate = inputSelectMessageTemplate
    this.bodyParamsInputs = bodyParamsInputs ?? []
    this.headerParamsInputs = headerParamsInputs ?? []
    this.buttonParamsInputs = buttonParamsInputs ?? []

    this.headerAssoc = []
    this.bodyAssoc = []
    this.buttonAssoc = []

    this.makeHeaderComponentAssoc()
    this.makeBodyComponentAssoc()
    this.makeButtonComponentAssoc()
  }

  makeHeaderComponentAssoc () {
    this.headerParamsInputs.forEach(select => {
      this.headerAssoc.push({
        key: select.dataset.key,
        value: select.value,
        type: select.dataset.type
      })
    })
  }

  makeBodyComponentAssoc () {
    this.bodyParamsInputs.forEach(select => {
      this.bodyAssoc.push({
        key: select.dataset.key,
        value: select.value,
        type: select.dataset.type ?? 'text'
      })
    })
  }

  makeButtonComponentAssoc () {
    this.buttonParamsInputs.forEach((btnWrapper, btnIndex) => {
      const params = []

      const buttonParams = btnWrapper.querySelectorAll('.button-param')

      if (buttonParams.length === 0) {
        return []
      }

      buttonParams.forEach(select => {
        params.push({
          key: select.dataset.key,
          value: select.value
        })
      })

      this.buttonAssoc.push({
        index: btnIndex,
        type: btnWrapper.dataset.btnType.toLowerCase(),
        params
      })
    })
  }

  getAssoc () {
    return {
      notification: this.inputSelectNotification.value,
      template: this.inputSelectMessageTemplate.value,
      components: {
        header: this.headerAssoc,
        body: this.bodyAssoc,
        button: this.buttonAssoc
      }
    }
  }
}

const notifFormSetup = new NotificationSetupForm(
  componentsRoot,
  headerComponentRoot,
  headerComponentParamsRoot,
  bodyComponentRoot,
  bodyComponentParamsRoot,
  buttonComponentRoot,
  buttonComponentParamsRoot,
  footerComponentRoot,
  footerComponentParamsRoot,
  btnSaveAssoc
)

/**
 * @param {MouseEvent} event
 */
async function handleMessageTemplateChange (event) {
  await notifFormSetup.reset()

  const templateCode = event.target.value
  const template = MESSAGE_TEMPLATES.find((template) => template.name === templateCode)

  const headerComponent = template.components.find(component => component.type === 'HEADER')
  const bodyComponent = template.components.find(component => component.type === 'BODY')
  const buttonComponents = template.components.find(component => component.type === 'BUTTONS')
  const footerComponent = template.components.find(component => component.type === 'FOOTER')

  if (headerComponent) {
    notifFormSetup.makeHeaderComponent(headerComponent)
  }

  notifFormSetup.makeBodyComplement(bodyComponent)

  if (buttonComponents) {
    notifFormSetup.makeButtonComponents(buttonComponents)
  }

  if (footerComponent) {
    notifFormSetup.makeFooterComponent(footerComponent.text)
  }
}

/**
 * @param {MouseEvent} event
 */
async function handleNotificationChange (event) {
  const notifCode = event.target.value

  if (notifCode === '') {
    notifFormSetup.reset()
    inputSelectMessageTemplate.value = ''
    inputSelectMessageTemplate.disabled = true

    return
  }

  const params = await request('get-notification-params', { notifName: notifCode })
    .then(res => res.json())
    .then(res => {
      if (!isEditMode) {
        inputSelectMessageTemplate.disabled = false
      }

      return res.data.paramsLabels
    })

  notifFormSetup.updateNotifParams(params)

  if (isEditMode) {
    inputSelectMessageTemplate.dispatchEvent(new Event('change'))
  }
}

/**
 * @param {MouseEvent} event
 */
function handleSaveAssocClick (event) {
  if (!wpAssociationForm.reportValidity()) {
    return
  }

  const assocGenerator = new AssocGenerator(
    inputSelectNotification,
    inputSelectMessageTemplate,
    document.querySelectorAll('.header-param'),
    document.querySelectorAll('.body-param'),
    document.querySelectorAll('.button-param-wrapper')
  )

  notifFormSetup.setLoadingState(true)

  const assoc = assocGenerator.getAssoc()

  const requestBody = {
    token: LKN_CSRF_TOKEN,
    data: { ...assoc }
  }
  const action = isEditMode ? 'wp-update-assoc' : 'wp-create-assoc'

  request(action, requestBody)
    .then(res => res.json())
    .then(res => {
      if (res.success) {
        if (isEditMode) {
          LknHookNotModal
            .setTitle(window.lknHn.lang.success)
            .setSimpleBody(window.lknHn.lang.notification_saved_success)
            .show()
        } else {
          const reason = res?.data?.reason ?? window.lknHn.lang.notification_saved_success

          LknHookNotModal
            .setTitle(window.lknHn.lang.success)
            .setSimpleBody(reason)
            .setSecondaryBtn(window.lknHn.lang.view_notification, res.data.editUrl)
            .show()

          setTimeout(() => window.location.reload(), 1200)
        }
      } else {
        LknHookNotModal
          .setTitle(window.lknHn.lang.error)
          .setSimpleBody(`${window.lknHn.lang.notification_save_failed} ${res.data.msg}`)
          .show()
      }
    })
    .catch(() => {
      LknHookNotModal
        .setTitle(window.lknHn.lang.error)
        .setSimpleBody(window.lknHn.lang.notification_save_failed)
        .show()
    })
    .finally(() => {
      notifFormSetup.setLoadingState(false)
    })
}

inputSelectNotification.addEventListener('change', handleNotificationChange)
inputSelectMessageTemplate.addEventListener('change', handleMessageTemplateChange)
btnSaveAssoc.addEventListener('click', handleSaveAssocClick)

if (isEditMode) {
  addEventListener('DOMContentLoaded', (event) => {
    inputSelectNotification.dispatchEvent(new Event('change'))
  })
} else if (inputSelectNotification.value !== '') {
  inputSelectNotification.dispatchEvent(new Event('change'))
}
